package com.codigoparallevar.minicards.parts.samples;

import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.graphics.Rect;
import android.util.Log;

import com.codigoparallevar.minicards.types.InputConnector;
import com.codigoparallevar.minicards.types.Moveable;
import com.codigoparallevar.minicards.types.OutputConnector;
import com.codigoparallevar.minicards.types.Part;
import com.codigoparallevar.minicards.types.PartGrid;
import com.codigoparallevar.minicards.types.RoundInputConnector;

import org.json.JSONException;
import org.json.JSONObject;

import java.util.Collections;
import java.util.LinkedList;
import java.util.List;

public class ColorBox implements Part {
    private final PartGrid _partGrid;
    private boolean _activated;
    private int _left;
    private int _top;
    private int _right;
    private int _bottom;
    private List<InputConnector> inputConnectors;
    private InputConnector _toggleInputConnector;

    public ColorBox(PartGrid partGrid, int left, int top, int right, int bottom) {
        _partGrid = partGrid;
        _left = left;
        _top = top;
        _right = right;
        _bottom = bottom;
        _activated = false;

        _toggleInputConnector = new RoundInputConnector(
                this,
                getInputConnectorCenterX(),
                getInputConnectorCenterY(),
                getInputConnectRadius());
        inputConnectors = new LinkedList<>();
        inputConnectors.add(_toggleInputConnector);
    }

    @Override
    public int getLeft() {
        return _left;
    }

    @Override
    public int getRight() {
        return _right;
    }

    @Override
    public int getTop() {
        return _top;
    }

    @Override
    public int getBottom() {
        return _bottom;
    }

    @Override
    public void draw(Canvas canvas, boolean devMode) {
        Paint paint = new Paint(Paint.ANTI_ALIAS_FLAG);
        if (_activated){
            paint.setColor(Color.GREEN);
        }
        else {
            paint.setColor(Color.RED);
        }

        if (devMode){
            drawConnector(canvas);
            // drawWires(canvas, devMode);
        }


        canvas.drawRect(
                new Rect(_left, _top,
                        _right, _bottom),
                paint);
    }

    private void drawConnector(Canvas canvas) {
        Paint connectorPaint = new Paint(Paint.ANTI_ALIAS_FLAG);
        connectorPaint.setColor(Color.YELLOW);

        canvas.drawCircle(
                getInputConnectorCenterX(), getInputConnectorCenterY(),
                getInputConnectRadius(),
                connectorPaint);
    }

    @Override
    public void moveEnd(int x, int y) {
        final int width = _right - _left;
        final int height = _bottom - _top;

        _left = x - width / 2;
        _right = _left + width;

        _top = y - height / 2;
        _bottom = _top + height;

        _toggleInputConnector.updatePosition(
                getInputConnectorCenterX(),
                getInputConnectorCenterY());
    }

    @Override
    public void drop(int x, int y) {
        moveEnd(x, y);
    }

    @Override
    public void touched() {
        Log.d("Placeholder", "Placeholder touched");
    }

    @Override
    public List<InputConnector> getInputConnectors() {
        return inputConnectors;
    }

    @Override
    public List<OutputConnector> getOutputConnectors() {
        return Collections.emptyList();
    }

    @Override
    public JSONObject serialize() throws JSONException {
        JSONObject serialized = new JSONObject();

        serialized.put("left", _left);
        serialized.put("top", _top);
        serialized.put("right", _right);
        serialized.put("bottom", _bottom);

        return serialized;
    }

    @Override
    public void sendSignal(RoundInputConnector roundInputConnector) {
        _activated = !_activated;
    }

    public static Part deserialize(PartGrid partGrid, JSONObject data) throws JSONException {
        int left = data.getInt("left");
        int top = data.getInt("top");
        int right = data.getInt("right");
        int bottom = data.getInt("bottom");

        return new ColorBox(partGrid, left, top, right, bottom);
    }

    @Override
    public boolean containsPoint(int x, int y) {
        return (x >= getLeft()) && (x <= getRight())
                && (y >= getTop()) && (y <= getBottom());
    }

    @Override
    public Moveable getMoveable() {
        return this;
    }

    public int getInputConnectorCenterX() {
        return getLeft();
    }

    private int getInputConnectRadius() {
        return (getRight() - getLeft()) / 2;
    }

    public int getInputConnectorCenterY() {
        return (getTop() + getBottom()) / 2;
    }
}
